#!/usr/bin/env python3

import boto3
import pprint
import json

""" SYSTEM INFO:

python3 -m pip freeze
    appdirs==1.4.4
    attrs==19.3.0
    black==19.10b0
    boto3==1.14.35
    botocore==1.17.35
    click==7.1.2
    docutils==0.15.2
    jmespath==0.10.0
    pathspec==0.8.0
    pkg-resources==0.0.0
    python-dateutil==2.8.1
    regex==2020.7.14
    s3transfer==0.3.3
    six==1.15.0
    toml==0.10.1
    typed-ast==1.4.1
    urllib3==1.25.10

lsb_release -a
    No LSB modules are available.
    Distributor ID: Ubuntu
    Description:    Ubuntu 19.10
    Release:        19.10
    Codename:       eoan
"""


def main():
    run_example(
        provider="Amazon",
        endpoint_url=None,
        aws_access_key_id="AWS-Access-Key",
        aws_secret_access_key="AWS-Secret-Key"
    )

    run_example(
        provider="Wasabi",
        endpoint_url="https://s3.us-east-1.wasabisys.com",
        aws_access_key_id="Wasabi-Access-Key",
        aws_secret_access_key="Wasabi-Secret-Key"
    )

    print("For diff run:\ncolordiff -u Amazon_response.txt Wasabi_response.txt")


def run_example(provider, endpoint_url, aws_access_key_id, aws_secret_access_key):
    bucket = "Enter-Bucket-Name"

    client = boto3.client(
        "s3",
        endpoint_url=endpoint_url,
        aws_access_key_id=aws_access_key_id,
        aws_secret_access_key=aws_secret_access_key,
    )

    client.create_bucket(Bucket=bucket)

    filenames = (
        "src/a/1",
        "src/a/3",
        "src/a/5",
        "src/b/1",
        # <------------- marker starts here
        "src/b/3",
        "src/b/5",
        # <------------- Amazon seeks to here and returns "src/c/" for NextMarker
        #                whereas Wasabi returns "src/b/" and causes loop
        "src/c/1",
        "src/c/3",
        "src/c/5",
        "src/d/1",
        "src/d/3",
        "src/d/5",
    )

    for filename in filenames:
        client.put_object(Bucket=bucket, Key=filename)

    response = client.list_objects(
        Bucket=bucket, Delimiter="/", Marker="src/b/2/", Prefix="src/", MaxKeys=1
    )
    del response["ResponseMetadata"]

    # The CommonPrefixes and NextMarker values are incorrect for Wasabi
    print(f"Output for {provider}:")
    pprint.pprint(response)
    # Save the response to file so that diff can be used to compare provider responses
    with open(f"{provider}_response.txt", "w") as outfile:
        pprint.pprint(response, stream=outfile)
    print("\n\n")

    # Delete the bucket and all its objects
    s3 = boto3.resource(
        "s3",
        endpoint_url=endpoint_url,
        aws_access_key_id=aws_access_key_id,
        aws_secret_access_key=aws_secret_access_key,
    )
    s3_bucket = s3.Bucket(bucket)
    s3_bucket.objects.delete()
    s3_bucket.delete()


if __name__ == "__main__":
    main()
